import 'package:json_annotation/json_annotation.dart';

part 'authenticate.g.dart';

@JsonSerializable()
class AuthenticateRequest {
  final Agent agent;
  final String username, password;
  final String? clientToken;
  final bool? requestUser;

  AuthenticateRequest(
      {required this.agent,
      required this.username,
      required this.password,
      this.clientToken,
      this.requestUser = true});

  factory AuthenticateRequest.fromJson(Map<String, dynamic> json) =>
      _$AuthenticateRequestFromJson(json);

  Map<String, dynamic> toJson() => _$AuthenticateRequestToJson(this);
}

@JsonSerializable()
class Agent {
  final String? name;
  final int version;

  Agent({this.name = "Minecraft", required this.version});
  factory Agent.fromJson(Map<String, dynamic> json) => _$AgentFromJson(json);

  Map<String, dynamic> toJson() => _$AgentToJson(this);
}

@JsonSerializable()
class AuthenticateResponse {
  final User user;
  final String clientToken, accessToken;
  final List<Profile> availableProfiles;
  final Profile selectedProfile;

  AuthenticateResponse(this.user, this.clientToken, this.accessToken,
      this.availableProfiles, this.selectedProfile);
  factory AuthenticateResponse.fromJson(Map<String, dynamic> json) =>
      _$AuthenticateResponseFromJson(json);
  Map<String, dynamic> toJson() => _$AuthenticateResponseToJson(this);
}

@JsonSerializable()
class Profile {
  final String name, id;

  Profile(this.name, this.id);
  factory Profile.fromJson(Map<String, dynamic> json) =>
      _$ProfileFromJson(json);
  Map<String, dynamic> toJson() => _$ProfileToJson(this);
}

@JsonSerializable()
class User {
  final String username, id;
  final List<Property> properties;

  User(this.username, this.id, this.properties);
  factory User.fromJson(Map<String, dynamic> json) => _$UserFromJson(json);
  Map<String, dynamic> toJson() => _$UserToJson(this);
}

@JsonSerializable()
class Property {
  final String name, value;

  Property(this.name, this.value);
  factory Property.fromJson(Map<String, dynamic> json) =>
      _$PropertyFromJson(json);
  Map<String, dynamic> toJson() => _$PropertyToJson(this);
}
